import { AbstractTreeNode, CHECKED, UNCHECKED, INDETERMINATE } from './abstract-node';
export class MultipleTreeNode extends AbstractTreeNode {
    constructor(origin, getId, opts) {
        super(origin);
        Object.defineProperty(this, "getId", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: getId
        });
        Object.defineProperty(this, "opts", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: opts
        });
        Object.defineProperty(this, "children", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_id", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_status", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_disabled", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this._id = getId(origin);
        this._disabled = !!(opts.disabled || origin.disabled);
        this._setUpChildren();
    }
    changeStatus(status, checkeds) {
        const newCheckeds = this._changeStatus(status, checkeds);
        return newCheckeds;
    }
    _calculateStatus(checkeds) {
        if (this.children) {
            let checkedNum = 0;
            let hasIndeterminate = false;
            let enabledLength = 0;
            for (const child of this.children) {
                if (child.enabled) {
                    enabledLength += 1;
                }
                if (child.checked) {
                    checkedNum += 1;
                }
                else if (child.enabled && child.indeterminate) {
                    hasIndeterminate = true;
                }
            }
            this._status = hasIndeterminate
                ? INDETERMINATE
                : checkedNum === 0
                    ? UNCHECKED
                    : checkedNum === enabledLength
                        ? CHECKED
                        : INDETERMINATE;
        }
        else {
            this._status = checkeds.has(this._id) ? CHECKED : UNCHECKED;
        }
    }
    _changeStatus(status, initValue) {
        let res = new Set(initValue);
        if (this.enabled && this._status !== status) {
            this._status = status;
            if (this.children) {
                this.children.forEach((child) => {
                    res = child._changeStatus(status, res);
                });
            }
            else {
                if (status === CHECKED) {
                    res.add(this._id);
                }
                else {
                    res.delete(this._id);
                }
            }
        }
        return res;
    }
    _setUpChildren() {
        if (this.origin.children) {
            this.children = this.origin.children.map((child) => {
                const node = new MultipleTreeNode(child, this.getId, Object.assign(this.opts, {
                    disabled: this._disabled,
                }));
                node._setParent(this);
                return node;
            });
        }
    }
}
