import { freeze, produce } from 'immer';
import { isFunction, isUndefined } from 'lodash';
import { useRef, useState } from 'react';
export function useDValue(initialValue, value, onChange, deepCompare, formControlInject) {
    const [_value, setValue] = useState(initialValue);
    const valueRef = useRef(_value);
    valueRef.current = isUndefined(formControlInject) ? (isUndefined(value) ? _value : value) : formControlInject[0];
    return [
        valueRef.current,
        (updater) => {
            const newValue = isFunction(updater) ? produce(valueRef.current, updater) : freeze(updater);
            const shouldUpdate = deepCompare ? !deepCompare(valueRef.current, newValue) : !Object.is(valueRef.current, newValue);
            if (shouldUpdate) {
                setValue(newValue);
                onChange === null || onChange === void 0 ? void 0 : onChange(newValue);
                if (!isUndefined(formControlInject)) {
                    formControlInject[1](newValue);
                }
            }
            return newValue;
        },
    ];
}
